from marshmallow import fields, ValidationError, Schema


class DatetimeFilter(fields.DateTime):
    """Field to filtering by __gte and __lte for mongoDB or __ge and __le for SQLAlchemy"""

    ValidateSuffixError = ValidationError(message={'errors': 'suffix must be _from or _until only'})

    def __init__(self, db_type, *args, **kwargs):
        if db_type not in ['sql', 'no_sql']:
            raise ValidationError(message={'errors': 'db_type must be sql or no_sql only'})

        if db_type == 'sql':
            self.from_value = '__ge'
            self.until_value = '__le'
        else:
            self.from_value = '__gte'
            self.until_value = '__lte'

        super().__init__(*args, **kwargs)

    def deserialize(self, value, attr, obj, **kwargs):
        if not self.attribute:

            if self.name.endswith('_from'):
                self.attribute = self.name[:-5] + self.from_value
            elif self.name.endswith('_until'):
                self.attribute = self.name[:-6] + self.until_value
            else:
                raise self.ValidateSuffixError

        if not self.attribute.endswith(self.until_value) and not self.attribute.endswith(self.from_value):
            raise self.ValidateSuffixError

        return super().deserialize(value, **kwargs)


if __name__ == '__main__':

    class Example(Schema):
        field_until = DatetimeFilter(db_type='sql')

    result = Example().load({'field_until': "2020-12-10T02:33:33Z"})
    print(result)


import typing
from collections import Iterable

from marshmallow import fields
from marshmallow.fields import missing_
from marshmallow.validate import OneOf


class Active(fields.Field):

    def __init__(self, *args, attribute: str = 'state', missing: typing.Any = missing_,
                 positives: Iterable = None,
                 negatives: Iterable = None,
                 as_filter: bool = False,
                 positive_filter=None,
                 negative_filter=None,
                 missing_filter=None,
                 **kwargs):
        self.filter_schema = as_filter
        positives = positives if positives else ['active']
        negatives = negatives if negatives else ['inactive']

        state_map = {}
        for positive in positives:
            state_map[positive] = True
        for negative in negatives:
            state_map[negative] = False
        self.string_state_map = state_map

        # for filter_schema
        missing = missing if missing and missing is not missing_ else ('missing' if self.filter_schema else missing_)
        if self.filter_schema:
            if positive_filter is not None:
                positive_filter = positive_filter
            elif len(positives) == 1:
                positive_filter = {attribute: positives[0]}
            else:
                positive_filter = {f'{attribute}__in': positives}

            if negative_filter is not None:
                negative_filter = negative_filter
            elif len(negatives) == 1:
                negative_filter = {attribute: negatives[0]}
            else:
                negative_filter = {f'{attribute}__in': negatives}

            self.filter_map = {
                'missing': {f'{attribute}__ne': 'deleted'} if missing_filter is None else missing_filter,
                'true': positive_filter,
                'false': negative_filter
            }
        else:
            self.bool_state_map = {v: k for k, v in self.string_state_map.items()}

        super().__init__(*args, attribute=attribute, missing=missing, **kwargs)

    def deserialize(self, value: typing.Any, *args, **kwargs):
        """Deserialize ``value``.

        :param value: The value to deserialize.
        """
        if value is missing_ and self.filter_schema and self.missing == 'missing':
            value = 'missing'
        return super().deserialize(value, *args, **kwargs)

    def _deserialize(self, value: typing.Any, *args, **kwargs):
        if self.filter_schema:
            OneOf(self.filter_map.keys())(value)
            if not (result_filter := self.filter_map[value]):
                return missing_
            [[self.attribute, result]] = result_filter.items()
            return result

        OneOf((True, False))(value)
        return self.bool_state_map.get(value, None)

    def _serialize(self, value: typing.Any, attr: str, obj: typing.Any, **kwargs):
        return self.string_state_map.get(value, None)



from marshmallow.fields import *
from .instance import Instance
from .active import Active


__all__ = ["Field", "Raw", "Nested", "Mapping", "Dict", "List", "Tuple", "String", "UUID", "Number", "Integer",
           "Decimal", "Boolean", "Float", "DateTime", "NaiveDateTime", "AwareDateTime", "Time", "Date", "TimeDelta",
           "Url", "URL", "Email", "IP", "IPv4", "IPv6", "Method", "Function", "Str", "Bool", "Int", "Constant",
           "Pluck", "Instance", "Active"]

__version__ = "1.2.2"



from unittest import main, TestCase

from marshmallow_advanced import Schema
from marshmallow_advanced.fields import Active


class ActiveFieldTests(TestCase):

    def test_simple_behaviour(self):
        class SimpleSchema(Schema):
            active = Active()

        result = SimpleSchema().dump({'state': 'active'})
        self.assertEqual(result, {'active': True})
        result = SimpleSchema().dump({'state': 'inactive'})
        self.assertEqual(result, {'active': False})
        result = SimpleSchema().dump({})
        self.assertEqual(result, {})

        result = SimpleSchema().load({'active': True})
        self.assertEqual(result, {'state': 'active'})
        result = SimpleSchema().load({'active': False})
        self.assertEqual(result, {'state': 'inactive'})
        result = SimpleSchema().load({})
        self.assertEqual(result, {})

    def test_advanced_behaviour(self):
        class AdvancedSchema(Schema):
            active = Active(positives=['active', 'running'], negatives=['inactive', 'stopped'])

        result = AdvancedSchema().dump({'state': 'active'})
        self.assertEqual(result, {'active': True})
        result = AdvancedSchema().dump({'state': 'running'})
        self.assertEqual(result, {'active': True})
        result = AdvancedSchema().dump({'state': 'inactive'})
        self.assertEqual(result, {'active': False})
        result = AdvancedSchema().dump({'state': 'stopped'})
        self.assertEqual(result, {'active': False})
        result = AdvancedSchema().dump({})
        self.assertEqual(result, {})

    def test_simple_filter_behaviour(self):
        class SimpleFilterSchema(Schema):
            active = Active(as_filter=True)

        result = SimpleFilterSchema().load({'active': 'true'})
        self.assertEqual(result, {'state': 'active'})
        result = SimpleFilterSchema().load({'active': 'false'})
        self.assertEqual(result, {'state': 'inactive'})
        result = SimpleFilterSchema().load({})
        self.assertEqual(result, {'state__ne': 'deleted'})

    def test_advanced_filter_behaviour(self):
        class AdvancedFilterSchema(Schema):
            active = Active(as_filter=True,
                            positives=['active', 'running'],
                            negatives=['inactive', 'stopped'])

        result = AdvancedFilterSchema().load({'active': 'true'})
        self.assertEqual(result, {'state__in': ['active', 'running']})
        result = AdvancedFilterSchema().load({'active': 'false'})
        self.assertEqual(result, {'state__in': ['inactive', 'stopped']})
        result = AdvancedFilterSchema().load({})
        self.assertEqual(result, {'state__ne': 'deleted'})

    def test_empty_extra_filter_fields_behaviour(self):
        class ExtendedFilterSchema(Schema):
            active = Active(as_filter=True, positive_filter={}, negative_filter={}, missing_filter={})

        result = ExtendedFilterSchema().load({'active': 'true'})
        self.assertEqual(result, {})
        result = ExtendedFilterSchema().load({'active': 'false'})
        self.assertEqual(result, {})
        result = ExtendedFilterSchema().load({})
        self.assertEqual(result, {})

    def test_extra_filter_fields_behaviour(self):
        class ExtendedFilterSchema(Schema):
            active = Active(as_filter=True,
                            positive_filter={'state__exists': True},
                            negative_filter={'state__exists': False},
                            missing_filter={'state__in': ['removed', 'deleted']})

        result = ExtendedFilterSchema().load({'active': 'true'})
        self.assertEqual(result, {'state__exists': True})
        result = ExtendedFilterSchema().load({'active': 'false'})
        self.assertEqual(result, {'state__exists': False})
        result = ExtendedFilterSchema().load({})
        self.assertEqual(result, {'state__in': ['removed', 'deleted']})


if __name__ == '__main__':
    main()



import pathlib
import setuptools
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(name='marshmallow_extended',
      version='1.2.2',
      url='https://github.com/blackacornlabs/marshmallow_advanced',
      long_description=README,
      description='Extended Marshmallow library',
      long_description_content_type="text/markdown",
      packages=setuptools.find_packages(),
      author='BlackAcornLabs',
      author_email='git@blackacorn.io',
      classifiers=[
          "Development Status :: 3 - Alpha",
          "Programming Language :: Python :: 3.8",
          "Intended Audience :: Developers",
          'Topic :: Software Development',
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      python_requires='>=3.8',
      zip_safe=False,
      install_requires=[
          "marshmallow~=3.8.0",
      ])
